//******************************************************************************
//
//   kintone Connector Program
//
//   Copyright (c) 2016 Cybozu
//
//******************************************************************************

using System;
using System.Xml;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Windows.Forms;
using System.Threading.Tasks;
using Newtonsoft.Json;

namespace CardConnections
{

    public partial class FormCardConnections : Form
    {
        [DllImport("User32.dll", EntryPoint = "SendMessage")]
        public static extern Int32 SendMessage(
                                            IntPtr hWnd,
                                            UInt32 Msg,
                                            UInt32 wParam,
                                            ref COPYDATASTRUCT lParam);

        [DllImport("kernel32.dll", EntryPoint = "GetPrivateProfileString")]
        public static extern Int32 GetPrivateProfileString(
                                            string lpAppName,
                                            string lpKeyName,
                                            string lpDefault,
                                            StringBuilder lpReturnedString,
                                            UInt32 nSize,
                                            string lpFileName);

        [DllImport("kernel32.dll", EntryPoint = "GetPrivateProfileInt")]
        public static extern UInt32 GetPrivateProfileInt(
                                            string lpAppName,
                                            string lpKeyName,
                                            Int32 nDefault,
                                            string lpFileName);

        /// <summary>
        /// COPYDATASTRUCT
        /// </summary>
        public struct COPYDATASTRUCT
        {
            public Int32 dwData;
            public Int32 cbData;
            public IntPtr lpData;
        }

        /// <summary>
        /// constructor
        /// </summary>
        public FormCardConnections()
        {
            InitializeComponent();

        }

        /// <summary>
        /// override
        /// </summary>
        /// <param name="winMessage">Windows message</param>
        protected override void WndProc(ref Message winMessage)
        {
            //switch (winMessage.Msg)
            if (winMessage.Msg == CardConnectionsMain.WM_COPYDATA)
            {
                // sendmessage
                //case CardConnectionsMain.WM_COPYDATA:
                string filePath;
                int result = 1;         //success
                try
                {
                    CardConnectionsMain.COPYDATASTRUCT cds = new CardConnectionsMain.COPYDATASTRUCT();
                    Type cdsType = cds.GetType();
                    cds = (CardConnectionsMain.COPYDATASTRUCT)winMessage.GetLParam(cdsType);
                    filePath = cds.lpData;

                    // DeleteDataFolder
                    Directory.Delete(Path.GetDirectoryName(CardConnectionsMain.ResultFilePath), true);
                    CardConnectionsMain.ResultFilePath = filePath;

                    // DeleteTmpFolder
                    Directory.Delete(CardConnectionsMain.TempDirectory, true);
                    // create
                    Directory.CreateDirectory(CardConnectionsMain.TempDirectory);

                    // clear file list
                    checkedListBox1.Items.Clear();
                    // copy file
                    CopyFile(filePath);
                }
                catch
                {
                    // Please describe processing when the error occurs.
                    result = 0;         // failed
                }
                winMessage.Result = (IntPtr)result;
                //break;
                //default:
                //break;
            }
            base.WndProc(ref winMessage);
        }

        /// <summary>
        /// Load
        /// </summary>
        private void FormCardConnections_Load(object sender, EventArgs e)
        {
            //if (String.IsNullOrEmpty(CardConnectionsMain.ResultFilePath) == false)
            if (!String.IsNullOrEmpty(CardConnectionsMain.ResultFilePath))
            {
                try
                {
                    // copy file
                    CopyFile(CardConnectionsMain.ResultFilePath);
                }
                catch
                {
                    // Please describe processing when the error occurs.
                }
            }
        }

        /// <summary>
        /// Click button_Close
        /// </summary>
        private void button_Close_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// analyze, and copy file
        /// </summary>
        /// <param name="resultFilePath">result file path</param>
        /// <returns>TRUE:SUCCESS,FALSE:ERROR</returns>
        ///
        private bool CopyFile(string resultFilePath)
        {
            int maxPath = 260;                  // MAXPATH
            StringBuilder returnedString = new StringBuilder(maxPath);

            UInt32 dataFileNum = GetPrivateProfileInt("FILES", "FileCount", 0, resultFilePath);     // Win32API
            if (dataFileNum == 0)
            {
                return false;
            }

            for (int i = 1; i <= dataFileNum; i++)
            {
                string keyName = "File" + i;
                string dataFilePath;
                string dataFileName;
                string destFileName;

                returnedString.Remove(0, returnedString.Length);
                GetPrivateProfileString("FILES", keyName, "", returnedString,
                                        (UInt32)(returnedString.Capacity), resultFilePath);         // Win32API
                dataFilePath = returnedString.ToString();
                dataFileName = Path.GetFileName(dataFilePath);
                destFileName = CardConnectionsMain.TempDirectory + @"\" + dataFileName;
                // Add to listBox
                checkedListBox1.Items.Add(dataFileName);
                checkedListBox1.SetItemChecked(i - 1, true);
                // copy data
                File.Copy(dataFilePath, destFileName);
            }
            return true;
        }

        public class MyBase64str
        {
            private Encoding enc;

            public MyBase64str(string encStr)
            {
                enc = Encoding.GetEncoding(encStr);
            }

            public string Encode(string str)
            {
                return Convert.ToBase64String(enc.GetBytes(str));
            }

            public string Decode(string str)
            {
                return enc.GetString(Convert.FromBase64String(str));
            }
        }
        /// <summary>
        /// XmlToJson
        /// </summary>
        public dynamic XmlToJson(string selectedFile)
        {
            //XMLからJSONに変換
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.LoadXml(File.ReadAllText(selectedFile));
            string jsoncarddata = JsonConvert.SerializeXmlNode(xmldoc);
            dynamic json = JsonConvert.DeserializeObject<dynamic>(jsoncarddata);
            return json.ContactXML.ContactXMLItem;
        }
        public static string EscapeString(string jsonstr)
        {
            System.Text.StringBuilder escapestr = new System.Text.StringBuilder(jsonstr);
            if (jsonstr != "")
            {
                escapestr.Replace("'", "\'");
                escapestr.Replace("\\", "\\\\");
                escapestr.Replace(@"""", @"\""");
            }
            return escapestr.ToString();
        }

        public class kintone
        {
            public string CreateBody(dynamic json, string appid, string filekey1, string filekey2, string loginname)
            {
                //JSONから必要なデータを取り出す
                string register = loginname;                                                                    //登録者
                string register_datetime = json["@lastModifiedDate"];                                           //登録日付
                string name = json.PersonName.PersonNameItem.FullName["#text"];                                 //氏名
                string name_pro = json.PersonName.PersonNameItem.FullName["@pronunciation"];                    //氏名フリガナ
                string company_name = json.Occupation.OccupationItem.OrganizationName["#text"];                 //会社名
                string company_name_pro = json.Occupation.OccupationItem.OrganizationName["@pronunciation"];    //会社名フリガナ
                string department = json.Occupation.OccupationItem.Department;                                  //部署名
                string occupation = json.Occupation.OccupationItem.JobTitle;                                    //役職名
                string country = json.Address.AddressItem.AddressLine[0]["#text"];                              //国_地域名
                string address = json.Address.AddressItem.FullAddress["#text"];                                 //住所
                string address_prefecture = json.Address.AddressItem.AddressLine[1]["#text"];                   //住所(都道府県)
                string address_city = json.Address.AddressItem.AddressLine[2]["#text"];                         //住所(市区町村)
                string address_number = json.Address.AddressItem.AddressLine[3]["#text"];                       //住所(番地以降)
                string postal_code = json.Address.AddressItem.AddressCode["#text"];                             //郵便番号
                string phonenum = json.Phone.PhoneItem[0]["#text"];                                             //電話番号
                string faxnum = json.Phone.PhoneItem[1]["#text"];                                               //FAX番号
                string mobilenum = json.Phone.PhoneItem[2]["#text"];                                            //携帯電話
                string url = json.Web.WebItem["#text"];                                                         //ＵＲＬ
                string email = json.Email.EmailItem["#text"];                                                   //電子メール
                string memo = json.Extension.ExtensionItem["#text"];                                            //メモ
                string body = @"
                                {
                                   app : " + appid + @",
                                   record:
                                   {
                                      ""登録者"": {""value"": [{""code"":""" + EscapeString(register) + @"""}]},
                                      ""登録日付"": {""value"": """ + EscapeString(register_datetime) + @"""},
                                      ""氏名"": { ""value"": """ + EscapeString(name) + @"""},
                                      ""氏名フリガナ"": { ""value"": """ + EscapeString(name_pro) + @"""},
                                      ""会社名"": { ""value"": """ + EscapeString(company_name) + @"""},
                                      ""会社名フリガナ"": { ""value"": """ + EscapeString(company_name_pro) + @"""},
                                      ""部署名"": { ""value"": """ + EscapeString(department) + @"""},
                                      ""役職名"": { ""value"": """ + EscapeString(occupation) + @"""},
                                      ""国_地域名"": { ""value"": """ + EscapeString(country) + @"""},
                                      ""住所"": { ""value"": """ + EscapeString(address) + @"""},
                                      ""都道府県"": { ""value"": """ + EscapeString(address_prefecture) + @"""},
                                      ""市区町村"": { ""value"": """ + EscapeString(address_city) + @"""},
                                      ""番地以降"": { ""value"": """ + EscapeString(address_number) + @"""},
                                      ""郵便番号"": { ""value"": """ + EscapeString(postal_code) + @"""},
                                      ""電話番号"": { ""value"": """ + EscapeString(phonenum) + @"""},
                                      ""携帯電話"": { ""value"": """ + EscapeString(mobilenum) + @"""},
                                      ""FAX番号"": { ""value"": """ + EscapeString(faxnum) + @"""},
                                      ""URL"": { ""value"": """ + EscapeString(url) + @"""},
                                      ""電子メール"": { ""value"": """ + EscapeString(email) + @"""},
                                      ""メモ"": { ""value"": """ + EscapeString(memo) + @"""},
                                      ""名刺画像データ"": { ""value"": [{fileKey:""" + filekey1 + @"""},{fileKey:""" + filekey2 + @"""}]}
                                    }
                                }";
                return body;
            }
            public async Task<HttpResponseMessage> GetFileKey(string filepass, string kintone_url, string apitoken)
            {
                using (FileStream fs = File.OpenRead(filepass))
                {
                    var fc = new FormCardConnections();
                    var httpClient = new HttpClient();
                    var content = new MultipartFormDataContent();
                    content.Add(new StreamContent(fs), "file", filepass);
                    httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("multipart/form-data"));
                    httpClient.DefaultRequestHeaders.Add("X-Cybozu-API-Token", apitoken);
                    HttpResponseMessage response = await httpClient.PostAsync(kintone_url, content);
                    return response;
                }
            }
            public async Task<HttpResponseMessage> PostCardData(string body, string host, string url, string apitoken)
            {
                Uri theUri = new Uri(url);
                var httpClient = new HttpClient();
                httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
                httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("multipart/form-data"));
                httpClient.DefaultRequestHeaders.Add("X-Cybozu-API-Token", apitoken);
                httpClient.DefaultRequestHeaders.Host = host;
                StringContent theContent = new StringContent(body, System.Text.Encoding.UTF8, "application/json");

                // データを送信します。
                var response = await httpClient.PostAsync(theUri, theContent);
                return response;
            }
        }

        private async void button_Import_Click(object sender, EventArgs e)
        {
            int count_items = checkedListBox1.CheckedItems.Count;
            if (count_items == 0)
            {
                log_richTextBox.Clear();
                log_richTextBox.Text += "kintoneに登録する名刺データにチェックを入れてください。";
                return;
            }
            button_Import.Enabled = false;
            log_richTextBox.Clear();

            int count_item = 1;
            log_richTextBox.Text += count_items + "件の名刺データを kintone のレコードに追加します。\r\n";
            log_richTextBox.Text += "----------------------------------------\r\n";
            log_richTextBox.Text += "件数 | ファイル名 | 氏名 | ステータス | レコード番号　\r\n\r\n";
            //初期設定
            string kintone_subdomain = "subdomain";//サブドメイン
            string kintone_record_url = "https://" + kintone_subdomain + ".cybozu.com/k/v1/record.json";
            string kintone_file_url = "https://" + kintone_subdomain + ".cybozu.com/k/v1/file.json";
            string kintone_host = kintone_subdomain + ".cybozu.com:443";
            string kintone_appid = "1000";                                       //Scansnap連携アプリのアプリ番号
            string kintone_apitoken = "d4n2055W4I333g55Z2134pJAhfJrlW433z53543l";//Scansnap連携アプリのAPIトークン
            string kintone_loginname = textBox_LoginName.Text;

            foreach (object carddata in checkedListBox1.CheckedItems)
            {
                //XMLからJSONに変換
                log_richTextBox.Text += count_item + "/" + count_items + "  ";
                var kintone = new kintone();
                string selectedFile = CardConnectionsMain.TempDirectory.ToString() + @"\" + carddata.ToString();
                var json = XmlToJson(selectedFile);

                //JSONから名刺画像データを取得
                log_richTextBox.Text += carddata.ToString() + "  " + json.PersonName.PersonNameItem.FullName["#text"] + "  ";
                string img_filepass_front = json.Image.ImageItem[0]["@url"];
                string img_filepass_back = json.Image.ImageItem[1]["@url"];
                Uri uri_img_front = new Uri(img_filepass_front);
                Uri uri_img_back = new Uri(img_filepass_back);
                img_filepass_front = uri_img_front.LocalPath;
                img_filepass_back = uri_img_back.LocalPath;
                try
                {
                    //名刺表面の画像ファイルのfilekeyの取得
                    HttpResponseMessage response_filekey_front = await kintone.GetFileKey(img_filepass_front, kintone_file_url, kintone_apitoken);
                    string response_content_front = await response_filekey_front.Content.ReadAsStringAsync();
                    if (!response_filekey_front.IsSuccessStatusCode)
                    {
                        log_richTextBox.Text += "\r\n" + response_content_front + "\r\n";
                        log_richTextBox.Text += count_item + "件目の名刺データで名刺画像ファイルのファイルキーの取得に失敗しました。処理を中断します。\r\n";
                        button_Import.Enabled = true;
                        return;
                    }
                    dynamic obj_front = JsonConvert.DeserializeObject<dynamic>(response_content_front);
                    string filekey_front = obj_front.fileKey;

                    //名刺裏面の画像ファイルのfilekeyの取得
                    HttpResponseMessage response_filekey_back = await kintone.GetFileKey(img_filepass_back, kintone_file_url, kintone_apitoken);
                    string response_content_back = await response_filekey_back.Content.ReadAsStringAsync();
                    if (!response_filekey_back.IsSuccessStatusCode)
                    {
                        log_richTextBox.Text += "\r\n" + response_content_back + "\r\n";
                        log_richTextBox.Text += count_item + "件目の名刺データで名刺画像ファイルのファイルキーの取得に失敗しました。処理を中断します。\r\n";
                        button_Import.Enabled = true;
                        return;
                    }
                    dynamic obj_back = JsonConvert.DeserializeObject<dynamic>(response_content_back);
                    string filekey_back = obj_back.fileKey;

                    //bodyの作成
                    string body = kintone.CreateBody(json, kintone_appid, filekey_front, filekey_back, kintone_loginname);
                    var response = await kintone.PostCardData(body, kintone_host, kintone_record_url, kintone_apitoken);
                    string content = await response.Content.ReadAsStringAsync();
                    if (!response.IsSuccessStatusCode)
                    {
                        log_richTextBox.Text += "Error: StatusCode" + response.StatusCode.ToString() + "\r\n" + content + "\r\n";
                        log_richTextBox.Text += count_item + "件目の名刺データの登録に失敗しました。処理を中断します。\r\n";
                        log_richTextBox.Text += "入力されたログイン名が正しいことを確認してください。";
                        button_Import.Enabled = true;
                        return;
                    }
                    dynamic obj_id = JsonConvert.DeserializeObject<dynamic>(content);
                    string recordId = obj_id.id;
                    log_richTextBox.Text += "  " + "Success" + "  " + recordId + "\r\n";
                    count_item += 1;
                }
                catch (Exception ex)
                {
                    log_richTextBox.Text += "Error:" + ex.Message + ex.HResult + "\r\n";
                    log_richTextBox.Text += count_item + "件目の名刺データの登録時に例外が検出されました。処理を中断します。";
                    button_Import.Enabled = true;
                    return;
                }
            }
            log_richTextBox.Text += "----------------------------------------\r\n";
            log_richTextBox.Text += (count_item - 1) + "件の名刺データの登録が完了しました。\r\n";
            log_richTextBox.Text += "名刺データ登録先アプリ：\r\n" + "https://" + kintone_subdomain + ".cybozu.com/k/" + kintone_appid + "\r\n";
            log_richTextBox.Text += "※続けて名刺をスキャンする場合は一度本アプリケーションを閉じてください。\r\n";
            log_richTextBox.DetectUrls = true;
            button_Import.Enabled = true;
        }
        private void button_check_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < checkedListBox1.Items.Count; i++)
            {
                checkedListBox1.SetItemChecked(i, true);
            }
        }
        private void button_uncheck_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < checkedListBox1.Items.Count; i++)
            {
                checkedListBox1.SetItemChecked(i, false);
            }
        }

        private void log_richTextBox_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(e.LinkText);
        }
    }
}
